using Oxide.Core;
using System.Collections.Generic;
using UnityEngine;
using System;

namespace Oxide.Plugins
{
    [Info("AutoClean", "Noviets", "1.0.5")]
    [Description("Provides automatic cleaning of objects outside of claimed areas")]

    public class AutoClean : HurtworldPlugin
    {
        private void Loaded()
        {
            permission.RegisterPermission("autoclean.admin", this);

            timer.Repeat(Convert.ToSingle(Config["UpdateIntervalSeconds"]), 0, () => { DoClean(); });

            if ((bool)Config["LoadPreviousDecayIntervals"])
            {
                ObjectList = Interface.GetMod().DataFileSystem.ReadObject<Dictionary<string, int>>("AutoClean/ObjectList");
            }
        }

        private Dictionary<string, int> ObjectList = new Dictionary<string, int>();

        private void SaveObjects() => Interface.GetMod().DataFileSystem.WriteObject("AutoClean/ObjectList", ObjectList);

        private OwnershipStakeServer stake;
        private int destroyed = 0;
        private int hasstake = 0;
        private int nostake = 0;
        private int num = 0;

        private void OnServerInitialized()
        {
            if ((bool)Config["AutoPruneLog"])
            {
                PruneLog();
            }
        }

        protected override void LoadDefaultConfig()
        {
            Config["UpdateIntervalSeconds"] = 7200;
            Config["IntervalsBeforeCleaning"] = 1;
            Config["ShowConsoleMessages"] = true;
            Config["LoadPreviousDecayIntervals"] = true;
            Config["AutoPruneLog"] = true;
            SaveConfig();
        }

        protected override void LoadDefaultMessages()
        {
            // Polish
            lang.RegisterMessages(new Dictionary<string, string>
            {
                ["prunecmd"] = "<color=aqua>[Sprzątacz]</color> Czyszczenie baz bez totemu!",
                ["prunesuccess"] = "<color=aqua>[Sprzątacz]</color> Czyszczenie baz bez totemu!",
                ["nopermission"] = "<color=aqua>[Sprzątacz]</color> Nie posiadasz uprawnień do tej komendy."
            }, this);
        }

        private string Msg(string msg, object steamID = null) => lang.GetMessage(msg, this, steamID.ToString());

        [ChatCommand("clean")]
        private void ManualClean(PlayerSession session)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "autoclean.admin") || session.IsAdmin)
            {
                DoClean(false);
                hurt.SendChatMessage(session, null, Msg("prunecmd", session.SteamId));
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }

        private void PruneLog()
        {
            List<string> existing = new List<string>();
            List<string> prune = new List<string>();
            int chked = 0;
            int pruned = 0;
            int safe = 0;
            foreach (GameObject obj in Resources.FindObjectsOfTypeAll<GameObject>())
            {
                uLink.NetworkView nwv = obj.GetComponent<uLink.NetworkView>();
                if (nwv != null && nwv.isActiveAndEnabled)
                {
                    existing.Add(nwv.viewID.id.ToString());
                }
            }
            foreach (string key in ObjectList.Keys)
            {
                chked++;
                if (existing.Contains(key))
                {
                    safe++;
                }
                else
                {
                    pruned++;
                    prune.Add(key);
                }
            }
            foreach (string s in prune)
            {
                ObjectList.Remove(s);
            }
            Puts("Checked: " + chked);
            Puts("Correct: " + safe);
            Puts("Pruned:  " + pruned);
            SaveObjects();
        }

        private void DoClean(bool automatic = true)
        {
            hasstake = 0;
            nostake = 0;
            destroyed = 0;
            num = 0;
            float t = 0;
            if (automatic)
            {
                hurt.BroadcastChat(null, Msg("prunesuccess"));
                t = 5f;
            }
            timer.Once(t, () =>
            {
                foreach (GameObject obj in Resources.FindObjectsOfTypeAll<GameObject>())
                {
                    if (obj.name.Contains("Constructed") || obj.name.Contains("StructureManager") || obj.name.Contains("Dynamic") || obj.name.Contains("LadderCollider(Clone)"))
                    {
                        var thecell = ConstructionUtilities.GetOwnershipCell(obj.transform.position);
                        if (thecell.ToString() != null)
                        {
                            var nView = uLink.NetworkView.Get(obj);
                            if (nView != null && nView.isActiveAndEnabled)
                            {
                                ConstructionManager.Instance.OwnershipCells.TryGetValue(ConstructionUtilities.GetOwnershipCell(obj.transform.position), out stake);
                                string ID = nView.viewID.id.ToString();
                                if (stake == null)
                                {
                                    nostake++;
                                    if (ObjectList.TryGetValue(ID, out num))
                                    {
                                        if (num == Convert.ToInt32(Config["IntervalsBeforeCleaning"]))
                                        {
                                            try
                                            {
                                                Singleton<HNetworkManager>.Instance.NetDestroy(nView);
                                                ObjectList.Remove(ID);
                                                destroyed++;
                                            }
                                            catch { }
                                        }
                                        else
                                        {
                                            ObjectList[ID] += 1;
                                        }
                                    }
                                    else
                                    {
                                        ObjectList.Add(ID, 1);
                                    }
                                }
                                else
                                {
                                    hasstake++;
                                    if (ObjectList.TryGetValue(ID, out num))
                                    {
                                        ObjectList.Remove(ID);
                                    }
                                }
                            }
                        }
                    }
                }
                SaveObjects();
                if ((bool)Config["ShowConsoleMessages"])
                {
                    Puts("Has Stake: " + hasstake);
                    Puts("No Stake : " + nostake);
                    Puts("Destroyed: " + destroyed);
                }
            });
        }
    }
}
